From 86bf46707055317d10eb8c843e2ae260df602b3c Mon Sep 17 00:00:00 2001
From: Simon Hailes <simon@screen.subtitling.com>
Date: Sun, 20 Sep 2009 22:58:16 +0100
Subject: [PATCH 2/2] Added notes on DDS (Display Definition Segment - for 'HD')

---
 modules/codec/dvbsub.c |   32 ++++++++++++++++++++++++++++++--
 1 files changed, 30 insertions(+), 2 deletions(-)

diff --git a/modules/codec/dvbsub.c b/modules/codec/dvbsub.c
index fea080c..ee41107 100644
--- a/modules/codec/dvbsub.c
+++ b/modules/codec/dvbsub.c
@@ -38,6 +38,27 @@
  * refers to the character index in the codepage belonging to the language specified
  * in the subtitle descriptor. Potentially it's designed for widechar
  * (but not for UTF-*) codepages.
+ *
+ *****************************************************************************
+ * Notes on DDS (Display Definition Segment)
+ * DDS (Display Definition Segment) tells the decoder how the subtitle image relates to
+ * the video image.
+ * For SD, the subtitle image is always considered to be for display at 720x576
+ * (although it's assumed that for NTSC, this is 720x480, this is not documented well)
+ * Also, for SD, the subtitle image is drawn 'on the glass' (i.e. after video scaling,
+ * letterbox, etc.)
+ * For 'HD' (subs marked type 0x14/0x24 in PSI), a DDS must be present, and the subs area
+ * is drawn onto the video area (scales if necessary).  The DDS tells the decoder what 
+ * resolution the subtitle images were intended for, and hence how to scale the subtitle 
+ * images for a particular video size  
+ * i.e. if HD video is presented as letterbox, the subs will
+ * be in the same place on the video as if the video was presented on an HD set
+ * indeed, if the HD video was pillarboxed by the decoder, the subs may be cut off as
+ * well as the video.  The intent here is that the subs can be placed accurately on the video
+ * - somthing which was missed in the original spec.
+ * 
+ * A DDS may also specify a window - this is where the subs images are moved so that the (0,0)
+ * origin of decode is offset.
  *****************************************************************************/
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -151,6 +172,7 @@ typedef struct dvbsub_display_s
     int                     i_height;
 
     bool              b_windowed;
+    /* these values are only relevant if windowed */
     int                     i_x;
     int                     i_y;
     int                     i_max_x;
@@ -982,6 +1004,8 @@ static void decode_display_definition( decoder_t *p_dec, bs_t *s )
 #endif
     }
 
+
+    /* see notes on DDS at top of file */
     /* Parse the display definition: */
     p_sys->display.i_version = i_version;
     p_sys->display.b_windowed = bs_read( s, 1 );
@@ -1004,7 +1028,7 @@ static void decode_display_definition( decoder_t *p_dec, bs_t *s )
     else
     {
         /* if not windowed, setup the window variables to good defaults */
-        /* not necessary, but.. */
+        /* not necessary, but to avoid future confusion.. */
         p_sys->display.i_x     = 0;
         p_sys->display.i_max_x = p_sys->display.i_width-1;
         p_sys->display.i_y     = 0;
@@ -1492,6 +1516,8 @@ static subpicture_t *render( decoder_t *p_dec )
         /* the window is within this... SO... we should leave i_original_picture_width etc. as is */
         /* and ONLY change i_base_x.  effectively i_max_x/y are only there to limit memory requirements*/
         /* we COULD use the upper limits to limit rendering to within these? */
+
+        /* see notes on DDS at the top of the file */
         i_base_x += p_sys->display.i_x;
         i_base_y += p_sys->display.i_y;
     }
@@ -2642,9 +2668,11 @@ static void encode_pixel_line_8bp( bs_t *s, subpicture_region_t *p_region,
 static void default_dds_init( decoder_t * p_dec )
 {
     decoder_sys_t *p_sys = p_dec->p_sys;
+    
+    /* see notes on DDS at the top of the file */
 
     /* configure for SD res in case DDS is not present */
-    p_sys->display.i_version = -1;
+    p_sys->display.i_version = 0xff; /* an invalid version so it's always different */
     p_sys->display.i_width = 720;
     p_sys->display.i_height = 576;
     p_sys->display.b_windowed = false;
-- 
1.6.4.msysgit.0

